#!/usr/bin/python3
import matplotlib as mpl
mpl.use('PS')
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import matplotlib.pyplot as plt
import numpy as np
from math import tanh, exp

cmap = cm.PiYG
lw = 0.1
mpl.rcParams['xtick.labelsize'] = mpl.rcParams['ytick.labelsize'] = 'x-small'

########################################################################
# Burgers' equation plots
########################################################################

def burg1(x,t):
  u = .5 - .5*tanh((x-.5*t-.25)/(4.*eps))
  return u

def burg2(x,t):
  a1 = (-x+.5-4.95*t)*.05/eps
  a2 = (-x+.5-0.75*t)*.25/eps
  a3 = (-x+.375)*.5/eps
  expa1 = expa2 = expa3 = 0.
  temp = max(a1,a2,a3)
  if (a1-temp) >= -35: expa1 = exp(a1-temp)
  if (a2-temp) >= -35: expa2 = exp(a2-temp)
  if (a3-temp) >= -35: expa3 = exp(a3-temp)
  u = (.1*expa1 + .5*expa2 + expa3) / (expa1+expa2+expa3)
  return u

vburg1 = np.vectorize(burg1)
vburg2 = np.vectorize(burg2)

step = .01
x = np.arange(0.,1.+step,step)
t = np.arange(0.,1.+step,step)
X, T = np.meshgrid(x, t)

for burg, eps, name in [(vburg1,1e-3,'B1e3'), (vburg1,1e-4,'B1e4'),
                        (vburg2,1e-3,'B2e3'), (vburg2,1e-4,'B2e4')]:
  fig = plt.figure()
  ax = fig.gca(projection='3d')
  ax.azim = -60
  ax.elev = 30
  U = np.reshape(burg(np.ravel(X), np.ravel(T)), X.shape)
  surf = ax.plot_surface(X, T, U, rstride=1, cstride=1, cmap=cmap, linewidth=lw)
  ax.set_xlabel('$x$')
  ax.set_ylabel('$t$')
  #ax.set_zlabel('$u(t,x)$') # this is vertical
  ax.text2D(1.0, 0.55, '$u(t,x)$', transform=ax.transAxes)
  plt.savefig(name+'.eps')
  plt.close()

########################################################################
# RCD plots
########################################################################

x = np.loadtxt('Xouts')
t = np.loadtxt('Touts')
X, T = np.meshgrid(x, t)
U = [np.loadtxt('Uouts'+str(i+1)) for i in range(4)]

for i, Ui in enumerate(U,start=1):
  fig = plt.figure()
  ax = fig.gca(projection='3d')
  ax.azim = 45
  ax.elev = 30
  surf = ax.plot_surface(X, T, Ui, rstride=1, cstride=1, cmap=cmap, linewidth=lw)
  ax.set_xlabel('$x$')
  ax.set_ylabel('$t$')
  #ax.set_zlabel('$u_'+str(i)+'(t,x)$') # this is vertical
  ax.text2D(-0.05, 0.5, '$u_'+str(i)+'(t,x)$', transform=ax.transAxes)
  plt.savefig('RCD'+str(i)+'.eps')
  plt.close()
