#!/usr/bin/python3
import matplotlib as mpl
mpl.use('PS')
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import matplotlib.pyplot as plt
import numpy as np
from math import tanh, exp
import sys

mpl.rcParams['xtick.labelsize'] = mpl.rcParams['ytick.labelsize'] = 'x-small'
mpl.rcParams['patch.edgecolor'] = 'black'
tweaks = {
  'lw': 0.5,
  'cmap': cm.coolwarm
}

########################################################################
# Burgers' equation plots
########################################################################
if len(sys.argv) > 1:

  X, T, U = np.loadtxt('Points001', unpack=True)
  fig = plt.figure()
  ax = fig.gca(projection='3d')
  ax.azim = -60
  ax.elev = 30
  surf = ax.plot_trisurf(X, T, U, **tweaks)
  ax.set_xlabel('$x$')
  ax.set_ylabel('$t$')
  #ax.set_zlabel('$u(t,x)$') # this is vertical
  ax.text2D(1.0, 0.55, '$u(t,x)$', transform=ax.transAxes)
  plt.savefig(sys.argv[1]+'.eps')
  plt.close()

########################################################################
# RCD plots
########################################################################
else:

  for i in range(1,5):
    X, T, U = np.loadtxt('Points%03d'%i, unpack=True)
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.azim = 45
    ax.elev = 30
    surf = ax.plot_trisurf(X, T, U, **tweaks)
    ax.set_xlabel('$x$')
    ax.set_ylabel('$t$')
    #ax.set_zlabel('$u_%d(t,x)$'%i) # this is vertical
    ax.text2D(-0.05, 0.5, '$u_%d(t,x)$'%i, transform=ax.transAxes)
    plt.savefig('RCD%d.eps'%i)
    plt.close()
