#!/usr/bin/python3
# this script fills a template LaTeX table with timing data
# it requires input parameters for the problem and table template
# jack pew, may 27, 2013

import glob
import sys
import re
import string
import numpy as np

template_file = 'template-md'

# we are making a table for this problem
try:
  prob = sys.argv[1]
except:
  prob = 'B1'

if 'B' in prob:
  try:
    e = sys.argv[2]
  except:
    e = '3'
  table_file = prob+'_e'+e+'_table_md.tex'
else:
  e = '0'
  table_file = prob+'_table_md.tex'


# the list of files containing program output
paths = glob.glob('timing/'+prob+'-k*e'+e+'-*.out')

# create the dictionary of data
data = {}
for path in paths:
  try:
    # find the dictionary key which corresponds to this data file
    kt, est = re.search(r'(k\d+t\d+)e\d-([slb])', path).groups()
    key = kt+est

    # open and read the file
    try:
      times = np.loadtxt(path, skiprows=1, ndmin=1)
    except ValueError:
      data[key] = '---'
      continue
    if not len(times):
      data[key] = '---'
      continue

    # insert data from this file
    data[key] = '{:.2f}'.format(times.mean())
  except:
    print(path)
    raise

# load in the template string
with open(template_file) as f:
  template = string.Template(f.read())

# fill the template with data
table = template.safe_substitute(data)

# write out the table
with open(table_file,'w') as f:
  f.write(table)
