#!/usr/bin/python3
# this script fills a template LaTeX table with machine independent data
# it requires input parameters for the problem and table template
# jack pew, may 27, 2013

import glob
import sys
import re
import string

template_file = 'template-mi'

# we are making a table for this problem
try:
  prob = sys.argv[1]
except:
  prob = 'B1'

if 'B' in prob:
  try:
    e = sys.argv[2]
  except:
    e = '3'
  table_file = prob+'_e'+e+'_table_mi.tex'
else:
  e = '0'
  table_file = prob+'_table_mi.tex'


# the list of files containing program output
paths = glob.glob('output/'+prob+'-k*e'+e+'-*.out')

# create the dictionary of data
data = {}
for path in paths:
  try:
    # find the pair of dictionary keys which correspond to this data file
    kt, est = re.search(r'(k\d+t\d+)e\d-([slb])', path).groups()
    key1 = kt+est+'1'
    key2 = kt+est+'2'

    # open and read the file
    with open(path) as f:
      contents = f.read()

    # search out IDID. test failed if IDID < 2
    IDID = int(re.search(r'IDID *= *(-?\d+)', contents).group(1))

    # if IDID < 2, mark the error and continue
    if IDID < 2:
      data[key1] = '---'
      data[key2] = '---'
      continue

    nint   = re.search(r'Final nint *= *(\d+)',     contents).group(1)
    steps  = re.search(r'ipar\(isteps\) *= *(\d+)', contents).group(1)
    remesh = re.search(r'nremesh *= *(\d+)',        contents).group(1)
    colds  = re.search(r'ipar\(icolds\) *= *(\d+)', contents).group(1)
    ncrdmp = re.search(r'ncrdmp *= *(\d+)',         contents).group(1)
    ncrslv = re.search(r'ncrlve *= *(\d+)',         contents).group(1)

    # insert data from this file
    data[key1] = nint+', '+steps+', '+remesh
    data[key2] = '('+colds+')'+' ['+ncrdmp+', '+ncrslv+']'
  except:
    print(path)
    raise

# load in the template string
with open(template_file) as f:
  template = string.Template(f.read())

# fill the template with data
table = template.safe_substitute(data)

# write out the table
with open(table_file,'w') as f:
  f.write(table)
